# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""
    This module provides various functions for managing Dexter contacts
"""

import os
import datetime
import vobject
import base64
import string
import threading

import gettext
from gettext import gettext as _
gettext.textdomain('dexter')

import dexter.backend.models as models


class ImportThread(threading.Thread):
    """a worker thread that parses a vcard string and puts the relevant data in a queue"""
    def __init__(self, paths, q):
        """create a new thread, q is a Queue.Queue object"""
        threading.Thread.__init__(self)
        self.paths = paths
        self.q = q
        
    def run(self):
        """parse the vcard string and fill the queue"""
        for path in self.paths:
            if self.q.request_stop:
                break
            vstring = open(path, 'r').read()
            import_from_vcard_string(vstring, self.q)

def export_to_vcard(contacts, path, dbus):
    """export a list of contacts to the vcard file specified by path"""
    GetEmails = dbus.get_dbus_method("GetEmails")
    GetPhones = dbus.get_dbus_method("GetPhones")
    GetAddresses = dbus.get_dbus_method("GetAddresses")
    GetNotes = dbus.get_dbus_method("GetNotes")
    card_text =""
    for contact in contacts:
        file_path = path
        card = vobject.vCard()
        card.add('n')
        card.n.value = vobject.vcard.Name(given=str(contact["first_name"]))
        if contact["last_name"] != "None":
            card.n.value.family = contact["last_name"]
        if contact["middle_name"] != "None":
            card.n.value.additional = contact["middle_name"]
        card.add('fn')
        card.fn.value = str(contact["full_name"])
        if contact["organization"] != "None":
            card.add('org')
            card.org.value = [str(contact["organization"])]
        if contact["birthday"] != "None":
            card.add('bday')
            card.bday.value = str(contact["birthday"])
        for email in GetEmails(int(contact["id"])):
            cardemail = card.add('email')
            cardemail.value = str(email[0])
            cardemail.type_param = str(email[1])
        for phone in GetPhones(int(contact["id"])):
            cardphone = card.add('tel')
            cardphone.value = str(phone[0])
            cardphone.type_param = str(phone[1])
        for address in GetAddresses(int(contact["id"])):
            cardadr = card.add('adr')
            cardadr.value = vobject.vcard.Address()
            if address["street"] != "None":
                cardadr.value.street=address["street"]
            if address["city"] != "None":
                cardadr.value.city=address["city"]
            if address["state"] != "None":
                cardadr.value.region=address["state"]
            if address["zip"] != "None":
                cardadr.value.code=address["zip"]
            cardadr.type_param = str(address["type"])
        for note in GetNotes(int(contact["id"])):
            cardnote = card.add('note')
            cardnote.value = str(note)
        if contact["photo"] != "None":
            b64 = base64.b64encode(open(models.IMAGE_DIR + \
                                        str(contact["photo"]), "rb").read())
            card.add("photo")
            card.photo.value = b64
            card.photo.encoding_param = "BASE64"
        card_text += card.serialize()
    FILE = open(file_path, 'w')
    FILE.write(card_text)
    FILE.close()


def import_from_vcard_string(vstring, q):
    """take a string read from a vcard file and parse it into contact data.
    
    Add each set of contact data to q so it can be retrieved and added to the db
    by the gobject.timeout_add in the parent thread.
    """
    vstring = string.translate(vstring, None, '\\')  # handle Apple's funky escaping
    components = vobject.readComponents(vstring)
    for card in components:
        if q.request_stop:
            break
        first_name = "None"
        last_name = "None"
        middle_name = "None"
        organization = "None"
        birthday = "None"
        photo_path = "None"
        notes = []
        emails = []
        addresses = []
        imnicks = []
        phones = []

        if hasattr(card, 'n'):
            if card.n.value.given:
                first_name = card.n.value.given.strip()      #
                last_name = card.n.value.family.strip()      #
                middle_name = card.n.value.additional.strip()#
            elif card.n.value.family:                        # Make sure we can use
                first_name = card.n.value.family.strip()     # SOMETHING for the first name
                last_name = card.n.value.family.strip()      #
                middle_name = card.n.value.additional.strip()#
            elif hasattr(card, 'email'):                     #
                first_name = card.email.value.strip()        #
            else:                                            #
                continue                                     # skip this contact if not
        elif hasattr(card, 'email'):                         #
            first_name = card.email.value.strip()            #
        else:                                                #
            continue                                         #
        if hasattr(card, 'org'):
            try:
                organization = " ".join(card.org.value).strip()
            except:
                organization = unicode(card.org.value).strip()
        if hasattr(card, 'bday'):
            birthdaystring = card.bday.value.strip()
            birthdaylist = birthdaystring.split("-")
            if birthdaylist[1] == u"00":
                birthdaylist[1] = u"01"
            if birthdaylist[2] == u"00":
                birthdaylist[2] = u"01"
            try:
                birthday = "-".join(birthdaylist)
            except:                                 # TODO: replace with proper
                print _("Could not import birthday") # error handling and logging
        if hasattr(card, 'photo'):
            if hasattr(card.photo, 'encoding_param'):
                try:
                    photo_name = datetime.datetime.now().strftime('%d%H%M%S%f')
                    photo_path = '/tmp/' + photo_name   # create a temp photo
                    photo_file = open(photo_path, 'w')  # file to be imported
                    photo_file.write(card.photo.value)  # by create_contact later
                    photo_file.close()
                except:
                    photo_path = "None"            # TODO: replace with proper 
                    print _("couldn't add photo")  # error handling and logging
        if hasattr(card, 'note'):
            notes.append(card.note.value.strip())
        for child in card.getChildren():
            if child.name == u'EMAIL':
                if hasattr(child, 'value'):
                    if hasattr(child, 'type_param'):
                        if len(child.params[u'TYPE']) > 1:
                            email_type = child.params[u'TYPE'][1].strip().title()
                        else:
                            email_type = child.params[u'TYPE'][0].strip().title()
                    else:
                        email_type = "Home"
                    emails.append((child.value.strip(), email_type))
            if child.name == u'ADR':
                if hasattr(child, 'value'):
                    adict = dict()
                    if hasattr(child, 'type_param'):
                        adict["type"] = child.type_param.title()
                    else:
                        adict["type"] = "Home"
                    adict["street"] = "".join(child.value.street)
                    adict["city"] = child.value.city.strip()
                    adict["state"] = child.value.region.strip()
                    adict["zip"] = child.value.code.strip()
                    addresses.append(adict)
            if child.name == u'TEL':
                if hasattr(child, 'value'):
                    if hasattr(child, 'type_param'):
                        phone_type = child.type_param.strip().title()
                    else:
                        phone_type = "Home"
                    phones.append((child.value.strip(), phone_type))
            if child.name == u'X-JABBER':
                if hasattr(child, 'value'):
                    nick_type = u"Jabber"
                    imnicks.append((child.value.strip(), nick_type))
        q.put([first_name, middle_name, last_name, organization, birthday, photo_path,
                emails, phones, addresses, imnicks, notes], True, 2)


def merge_contacts(contacts, dbus):
    GetEmails = dbus.get_dbus_method("GetEmails")
    GetPhones = dbus.get_dbus_method("GetPhones")
    GetAddresses = dbus.get_dbus_method("GetAddresses")
    GetIMNicks = dbus.get_dbus_method("GetIMNicks")
    GetNotes = dbus.get_dbus_method("GetNotes")
    CreateContact = dbus.get_dbus_method("CreateContact", dbus_interface='org.elementary.dexterserver')
    DeleteContact = dbus.get_dbus_method("DeleteContact")
    firstnames = []
    middlenames = []
    lastnames = []
    organizations = []
    birthdays = []
    photo_path = "None"
    emails = []
    phones = []
    addresses = []
    imnicks = []
    note = u""
    for contact in contacts:
        if contact["first_name"] != "None":
            if not contact["first_name"] in firstnames:
                firstnames.append(contact["first_name"])
        if contact["middle_name"] != "None":
            if not contact["middle_name"] in middlenames:
                middlenames.append(contact["middle_name"])
        if contact["last_name"] != "None":
            if not contact["last_name"] in lastnames:
                lastnames.append(contact["last_name"])
        if contact["organization"] != "None":
            if not contact["organization"] in organizations:
                organizations.append(contact["organization"])
        if contact["birthday"] != "None":
            if not contact["birthday"] in birthdays:
                birthdays.append(contact["birthday"])
        if contact["photo"] != "None":
            if photo_path == "None":
                photo_path = models.IMAGE_DIR + contact["photo"]
            else:
                thissize = os.path.getsize(models.IMAGE_DIR + contact["photo"])
                oldsize = os.path.getsize(photo_path)
                if thissize > oldsize:
                    photo_path = models.IMAGE_DIR + contact["photo"]
        for email in GetEmails(int(contact["id"])):
            email_exists = False
            for item in emails:
                if email[0] == item[0]:
                    email_exists = True
                    if item[1] == "Internet":
                        emails.remove(item)
                        email_exists = False
            if not email_exists:
                emails.append(email)
        for phone in GetPhones(int(contact["id"])):
            phone_exists = False
            for item in phones:
                if phone[0] == item[0]:
                    phone_exists = True
            if not phone_exists:
                phones.append(phone)
        for address in GetAddresses(int(contact["id"])):
            address_exists = False
            for item in addresses:
                if address["street"] == item["street"]:
                    address_exists = True
            if not address_exists:
                addresses.append(address)
        for imnick in GetIMNicks(int(contact["id"])):
            imnick_exists = False
            for item in imnicks:
                if imnick[0] == item[0]:
                    imnick_exists = True
            if not imnick_exists:
                imnicks.append(imnick)
        for anote in GetNotes(int(contact["id"])):
            if anote and anote.strip() != "None" and anote.strip() != "":
                note += (anote.strip() + "\n")
        DeleteContact(int(contact["id"]))
    if len(firstnames) > 1 or len(middlenames) > 1 or len(lastnames) > 1 or \
                                len(organizations) > 1 or len(birthdays) > 1:
        data = {"first" : firstnames, "middle" : middlenames, "last" : lastnames,
                "org" : organizations, "birthday" : birthdays,
                "photo" : photo_path, "emails" : emails, "phones" : phones,
                "addresses" : addresses, "imnicks" : imnicks, "notes" : [note]}
        return data
    else:
        if not firstnames:
            firstnames = ["None"]
        if not middlenames:
            middlenames = ["None"]
        if not lastnames:
            lastnames = ["None"]
        if not organizations:
            organizations = ["None"]
        if not birthdays:
            birthdays = ["None"]
        new_id = CreateContact(firstnames[0], middlenames[0], lastnames[0], 
                               organizations[0], birthdays[0], photo_path,
                               emails, phones, addresses, imnicks, [note])
        return int(new_id)
